// SPDX-FileCopyrightText: 2025 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later
#include "arrayOfColor.h"

void registerArrayOfColorMetaType()
{
    qRegisterMetaType<ArrayOfColor>("ArrayOfColor");
    qDBusRegisterMetaType<ArrayOfColor>();
}
