/****************************************************************************
** Meta object code from reading C++ file 'appearancedbusproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../appearancedbusproxy.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'appearancedbusproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19AppearanceDBusProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto AppearanceDBusProxy::qt_create_metaobjectdata<qt_meta_tag_ZN19AppearanceDBusProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppearanceDBusProxy",
        "PrimaryChanged",
        "",
        "Primary",
        "MonitorsChanged",
        "QList<QDBusObjectPath>",
        "monitors",
        "DesktopBackgroundsChanged",
        "desktopBackgrounds",
        "GreeterBackgroundChanged",
        "greeterBackground",
        "HandleForSleep",
        "sleep",
        "WallpaperURlsChanged",
        "GetCurrentWorkspace"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'PrimaryChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'MonitorsChanged'
        QtMocHelpers::SignalData<void(QList<QDBusObjectPath>)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'DesktopBackgroundsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 8 },
        }}),
        // Signal 'GreeterBackgroundChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'HandleForSleep'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'WallpaperURlsChanged'
        QtMocHelpers::SignalData<void(QString) const>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'GetCurrentWorkspace'
        QtMocHelpers::SlotData<int()>(14, 2, QMC::AccessPublic, QMetaType::Int),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AppearanceDBusProxy, qt_meta_tag_ZN19AppearanceDBusProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AppearanceDBusProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AppearanceDBusProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AppearanceDBusProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19AppearanceDBusProxyE_t>.metaTypes,
    nullptr
} };

void AppearanceDBusProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppearanceDBusProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->PrimaryChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->MonitorsChanged((*reinterpret_cast<std::add_pointer_t<QList<QDBusObjectPath>>>(_a[1]))); break;
        case 2: _t->DesktopBackgroundsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->GreeterBackgroundChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->HandleForSleep((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->WallpaperURlsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: { int _r = _t->GetCurrentWorkspace();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QDBusObjectPath> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AppearanceDBusProxy::*)(const QString & )>(_a, &AppearanceDBusProxy::PrimaryChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppearanceDBusProxy::*)(QList<QDBusObjectPath> )>(_a, &AppearanceDBusProxy::MonitorsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppearanceDBusProxy::*)(const QStringList & )>(_a, &AppearanceDBusProxy::DesktopBackgroundsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppearanceDBusProxy::*)(const QString & )>(_a, &AppearanceDBusProxy::GreeterBackgroundChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppearanceDBusProxy::*)(bool )>(_a, &AppearanceDBusProxy::HandleForSleep, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppearanceDBusProxy::*)(QString ) const>(_a, &AppearanceDBusProxy::WallpaperURlsChanged, 5))
            return;
    }
}

const QMetaObject *AppearanceDBusProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AppearanceDBusProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19AppearanceDBusProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AppearanceDBusProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void AppearanceDBusProxy::PrimaryChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AppearanceDBusProxy::MonitorsChanged(QList<QDBusObjectPath> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AppearanceDBusProxy::DesktopBackgroundsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AppearanceDBusProxy::GreeterBackgroundChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void AppearanceDBusProxy::HandleForSleep(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void AppearanceDBusProxy::WallpaperURlsChanged(QString _t1)const
{
    QMetaObject::activate<void>(const_cast< AppearanceDBusProxy *>(this), &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
